#ifndef _DLCartesian_H_
#define _DLCartesian_H_

#include "DLImage.h"
#include "DLPoint.h"
#include "DLBitsPerPixelConverter.h"

#include <cmath>

/**
 * \brief Cartesian Image Calculation.
 *
 * The algorithm is implemented as a static function, so no object of
 * class DLCartesian needs to be instantiated. See below for a usage example.
 */
class DLCartesian
{
   public:
   /** \cond **/
   /**
    * Default DLCartesian Constructor
    */
   DLCartesian();

   /**
    * Default DLCartesian Destructor
    */
   virtual ~DLCartesian();
   /** \endcond **/

   /**
    * Compute the cartesian image of a polar image.
    * Assumes that the origin of the polar image is located
    * at the center of the input image.
    * The following code example will
    * return the cartesian image for <i>test.tif</i>:
    * \code
    * DLImage i ("test.tif");
    * DLImage cartesianImage = DLCartesian::dlCartesianImage(i, true);
    * \endcode
    * \param  in const DLImage &
    * \return output DLImage
    */
   static DLImage dlCartesianImage ( const DLImage & in );

   private:

   static DLImage dlCartesianImage ( const DLImage & in, DLPoint centroid, double rmax );

   static DLImage dlCartesianImageBinary ( const DLImage & in, DLPoint centroid, double rmax );
   static DLImage dlCartesianImageGray ( const DLImage & in, DLPoint centroid, double rmax );
   static DLImage dlCartesianImageColor ( const DLImage & in, DLPoint centroid, double rmax );



};//end class

#endif //_DLCartesian_H_
