#ifndef DLCONNECTEDCOMPONENTSUTILS_H
#define DLCONNECTEDCOMPONENTSUTILS_H

//#include "DLCcConst.h"
#include "DLTypes.h"
#include "DLLogger.h"
#include "DLDatarun.h"
#include "DLPoint.h"
#include "DLException.h"

#ifndef DOXYGEN_SHOULD_SKIP_THIS 
///Temporary  data structures used in program to store data runs 
typedef struct {
   friend class DLConnectedComponents;
   friend class DLConnectedComponentsUtils;
   private:
   ///row of run   
   int run_row;                       
   ///first pixel of run 
   int run_start;                      
   ///last+1 pixel of run
   int run_stop;                       
   ///link runs  same component 
   int run_link;                     
} DLtmpdatalink;


/// Temporary Data Structure to store a document 
typedef struct {
   friend class DLConnectedComponents;
   friend class DLConnectedComponentsUtils;

   private:
   ///unique row flag
   int urow_flag;          
   ///mark current components
   int *still_curr;  
   ///mark prev runs still curr		
   int *B_Seen;                       
   /// Row
   int row;
   ///mast to invert Image bytes
   int mask[256];                     
   ///ptr to LUT for Image bytes
   int *ptr[256];                     
   ///pos of runstarts in bytes
   int LUT[5*256];                    
   ///for patching end of rows
   unsigned char msk[3];                      
   ///extra end of row bits
   int extra_bits;                    
   ///free tmp component numbers
   int *num_stk;                      
   ///ptr in run length array
   int irun;                          
  
   ///link list for runs 
   DLtmpdatalink *R;   
   /// Rhead
   int *Rhead;
   ///Rtail
   int *Rtail;
   ///Rlink
   int *Rlink;
   ///next Rlink
   int next_Rlink;
   ///free R
   int free_R;
   ///top R
   int top_R;
  
   ///link list for concavities 
   DLtmpdatalink *C;                  
   ///Chead
   int *Chead;
   ///Ctail
   int *Ctail;
   ///Clink
   int *Clink;
   ///next clink
   int next_Clink;
   ///free C
   int free_C;
   ///top C
   int top_C;
} DLtmpdocdata;


///Temp Runs data structure
typedef struct {   
   friend class DLConnectedComponents;
   friend class DLConnectedComponentsUtils;

   private:
   ///runstarts on row
   int *run;                         
   ///run component number
   int *num;                        
   ///run index 
   int i;                            
   ///number of runs on line
   int len;                        
} DLtmpdata;

/////Point Data
//typedef struct { 
//	friend class DLConnectedComponents;
//	friend class DLConnectedComponentsUtils;
//private:
//	///X value
//  int  x;
//  ///Y value
//  int  y;
//} DLpoint;

/////Doclib Data run data structure
//typedef struct {
//	friend class DLConnectedComponents;
//	friend class DLConnectedComponentsUtils;
//private:
//	///row of run 
//  int run_row;        
//  ///first pixel of run	
//  int run_start;                    
//  ///last+1 pixel of run 
//  int run_stop;                      
//} DLdataruns;


///Structure containing the Raw CC list

typedef struct {
   friend class DLConnectedComponents;
   friend class DLConnectedComponentsUtils;
   private:
   ///num of pixels in region
   int area;                  
   ///row centroid  
   float rcentroid;                 
   ///col centroid
   float ccentroid;                  
   ///length of perimeter 
   int perimeter;                    
   ///bounding box extents 
   int rmin ;                        
   ///bounding box extents 
   int rmax ;                        
   ///bounding box extents
   int cmin ;                        
   ///bounding box extents
   int cmax ;                        
   ///row index of seed pixel 
   int rseed;                        
   ///col index of seed pixel
   int cseed ;                       
   ///number of holes in component
   int number_of_holes;             
   ///number of runs in seg
   int num_runs;                     
   ///list of runs
   DLDatarun *runs;                 
   ///number of up concavities
   int num_conc_up;                  
   ///list of up concavitys 
   DLDatarun *conc_up;              
   ///number of down concavities
   int num_conc_down;                
   ///list of down concavitys
   DLDatarun *conc_down;            
   ///num_points_convex_hull
   int num_points_convex_hull;       
   ///points on convex_hull
   DLPoint *convex_hull;             
   ///area_of_convex_hull
   float area_of_convex_hull;        
   ///perimeter_of_convex_hull
   float perimeter_of_convex_hull;   
   ///first 14 moments
   float moments[14];                
   ///link to user defined data
   int *user_data;                   
} DLCCcomponent;


///Document that contains a list of RAW CC
typedef struct {
   friend class DLConnectedComponents;
   friend class DLConnectedComponentsUtils;
   private:
   ///filename of the document
   char cFileName[FILENAME_MAX];      
   ///number of columns in document
   int iCols;                         
   ///number of rows
   int iRows;                        
   ///(iCols + 7)/8  the number of bytes across
   int iColBytes;                     
   ///image
   short *image;                      
///data on  components
   DLCCcomponent* ComponentList;  
   ///number of  components
   int iComponentCount;               
   ///length of S          
   int top_S;                         
   ///stack pointer for next seg num
   int next_num;                     
   ///skew angle in radians
   double dSkewAngle;                
   ///link to user defined data
   int *user_data;                   
   ///length of image array
   int iImageLength;                 
   ///flag - 0/1 if 1 calculate moments
   int iCalcMoments;                 
   ///flag - 0/1 if 1 calculate convex/concave hulls
   int iCalcHulls;                   
   ///glad2tif libraries amount to fix dis errors 
   int iFixErrors;                    
   ///find the skew of the document, only good for 0->(10 15) degrees 
   int iCalcSkew;                     
   ///horizontal dpi, set by tbOpen
   float fHorizontalRes;                
   ///vertical dpi, set by tbOpen
   float fVerticalRes;                  
   ///bits per byte, either 8 or 1
   int iPixelDensity;                 
   ///enable warning messages to the screen
   DL_SHORT sEnableWarnings;             
   ///black/white image, 8 pixels per byte, has padding
   DL_UBYTE* ub1BitImage;              
   ///gray/color image, 1 pixel per byte, no padding needed
   DL_UBYTE* ub8BitImage;              
} DLCCdocument;


/**
 * ConnectedComponentsUtils is a class to process all connectedComponent
 * operations.
 */
class DLConnectedComponentsUtils
{
   friend class DLConnectedComponents;
   private:

   /**
    *  DLConnectedComponentsUtils Copy  Constructor
    */
   DLConnectedComponentsUtils(const DLConnectedComponentsUtils & orig);

   /**
    *  Asignment operator
    * \param right DLConnectedComponentsUtils 
    */
   const DLConnectedComponentsUtils & operator=(const DLConnectedComponentsUtils & right);
	

   /**
    * Calculate connected components
    * \param D1 DLCCdocument to store the returned CC
    * \return int of error code
    */
   static void DLCalcConnectedComponents(DLCCdocument * D1);
	
   /**
    * Clean up DLCCdocument and free up memory
    * \param doink DLCCdocument to be cleaned
    * \return int of error code
    */
   static void DLClose(DLCCdocument* doink);

   /**
    * Free up CC in DLCCdocument
    * \param doink DLCCdocument to be cleaned
    * \return int of error code
    */
   static int DLFreeConnectedComponents(DLCCdocument* doink);

   /**
    * Free up CC in DLCCdocument
    * \param lampimage 
    * \param w width
    * \param bw 
    * \param h height
    * \param outstarts 
    * \return int of error code
    */
   static int DLiPixelToRunStart(unsigned char *lampimage,int w,int bw,int h,short **outstarts);
   /**
    * Default DLConnectedComponentsUtils constructor
    */
   DLConnectedComponentsUtils(void);
   /**
    * Default DLConnectedComponentsUtils destructor
    */	
   virtual ~DLConnectedComponentsUtils(void);

   /**
    * Gets Data runs
    * \param D DLtmpdocdata
    * \param A DLtmpdocdata
    * \param T DLtmpdocdata
    */
   static void GET_RUNS(DLCCdocument * D, DLtmpdata * A, DLtmpdocdata * T);
	
   /**
    * Gets Data runs
    * \param H DLCCdocument
    * \param D DLCCdocument
    * \param A DLtmpdata
    * \param T DLtmpdocdata
    */
   static void NEW_SEGMENT(DLCCdocument * H, DLCCdocument * D, DLtmpdata * A, DLtmpdocdata * T);
   /**
    * Gets Data runs
    * \param H DLCCdocument
    * \param A DLCCdocument
    * \param B DLtmpdata
    * \param T DLtmpdocdata
    */
   static void ADD_RUN(DLCCdocument * H, DLtmpdata * A, DLtmpdata * B, DLtmpdocdata * T);
   /**
    * Gets Data runs
    * \param H DLCCdocument
    * \param D DLtmpdata
    * \param B DLtmpdata
    * \param T DLtmpdocdata
    */
   static void MERGE(DLCCdocument * H, DLCCdocument * D,
                     DLtmpdata * A, DLtmpdata * B, DLtmpdocdata * T);
   /**
    * Gets Data runs
    * \param H DLCCdocument
    * \param A DLtmpdata
    * \param B DLtmpdata
    */
   static void OVERLAP(DLCCdocument * H, DLtmpdata * A, DLtmpdata * B);
   /**
    * Gets Data runs
    * \param H DLCCdocument
    * \param D DLCCdocument
    * \param T DLtmpdocdata
    * \param A DLtmpdata
    * \param B DLtmpdata
    */
   static void OUTPUT(DLCCdocument * H, DLCCdocument * D, DLtmpdocdata * T,
                      DLtmpdata * A, DLtmpdata * B);
   /**
    * Gets Data runs
    * \param D DLCCdocument
    * \param B DLtmpdata
    * \param T DLtmpdocdata
    */
   static void GET_MEMORY_S(DLCCdocument * D, DLtmpdata * B, DLtmpdocdata * T);
   /**
    * Gets Data runs
    * \param D DLCCdocument
    * \param A DLtmpdata
    * \param T DLtmpdocdata
    */
   static void GET_MEMORY_RC(DLCCdocument * D, DLtmpdata * A, DLtmpdocdata * T);
   /**
    * Gets Data runs
    * \param H DLCCdocument
    * \param T DLtmpdocdata
    * \param n int
    * \param run_row int
    * \param run_start int
    * \param run_stop int
    */
   static void ADD_CONCAVITY(DLCCdocument * H, DLtmpdocdata * T, 
                             int n, int run_row, int run_start, int run_stop);
};
#endif //DOXYGEN_SHOULD_SKIP_THIS
#endif // DLCONNECTEDCOMPONENTSUTILS_H
