#ifndef _DLDilate_H_
#define _DLDilate_H_

#include "DLImage.h"
#include "DLPoint.h"
#include "DLBitsPerPixelConverter.h"

#include <cmath>

/**
 * \brief Dilate an image.
 */
class DLDilate
{
   public:
   /** \cond **/
   /**
    * Default DLDilate Constructor
    */
   DLDilate();

   /**
    * Default DLDilate Destructor
    */
   virtual ~DLDilate();
   /** \endcond **/

   /** 
    * Dilate an image.
    * 
    * This function operates only on binary (B&W) images.
    * At each application of the dilate operation, a white pixel 
    * is made black if any of its neighbors in the four principal
    * directions is black.
    * \param image binary DLImage to dilate
    * \param numberOfTimes number of times to apply dilate operation
    * \return dilated image
    */	
   static DLImage dlDilate ( const DLImage & image, int numberOfTimes );

   private:
   static DLImage dlDilateGray ( const DLImage & image, int numberOfTimes );
   static DLImage dlDilateBinary ( const DLImage & image, int numberOfTimes );

};//end class

#endif //_DLDilate_H_
