#ifndef _DLHistogram_H_
#define _DLHistogram_H_

#include "DLImage.h"
#include "DLBitsPerPixelConverter.h"

#include <cmath>

/**
 * \brief Calculate the Histogram of an image.
 */
class DLHistogram
{
   public:
   /** \cond **/
   /**
    * Default DLHistogram Constructor
    */
   DLHistogram();

   /**
    * Default DLHistogram Destructor
    */
   ~DLHistogram();
   /** \endcond **/

   /**
    * Computes a histogram of pixel values for a binary or grayscale image.
    * The function returns the histogram as an STL vector;
    * the first element records the number of pixels with pixel value 0,
    * and so on. For binary images, the histogram vector will contain 
    * two elements, and for grayscale images, the histogram vector will 
    * contain 256. 
    * \param image DLImage(bw or gray) to generate histogram for
    * \return Histogram data
    */
   static vector<int> dlHistogramImage ( const DLImage & image );

   private:

};//end class

#endif //_DLHistogram_H_
