#ifndef _DLMask_H_
#define _DLMask_H_

#include "DLImage.h"
#include "DLPoint.h"
#include "DLBitsPerPixelConverter.h"

#include <cmath>

/**
 * \brief Mask an image.
 */
class DLMask
{
   public:
   /** \cond **/
   /**
    * Default DLMask Constructor
    */
   DLMask();

   /**
    * Default DLMask Destructor
    */
   virtual ~DLMask();
   /** \endcond **/

   /**
    * Mask an image.
    *
    * Convolves the image with a square (<i>N</i> x <i>N</i>) filter.
    * The filter should be provided as a row-major 2-dimensional \c double
    * array. For example:
    * \code
    * DLImage i("test.jpg");
    * // create masking filter: use array[][] format for ease of initialization
    * double maskarray[3][3] = 
    *   { {1,  1,  1},
    *     {1, -8,  1},
    *     {1,  1,  1}  };
    *
    * // convert double[][] into double** for input to dlMask() 
    * double ** mask = new double*[3];
    * for (int qq=0; qq<3; qq++) {
    *   mask[qq] = &maskarray[qq][0];
    * }
    *
    * // perform masking operation
    * DLImage outim = DLMask::dlMask(i,mask,3);
    * \endcode
    * \param image DLImage to be masked
    * \param mask array containing masking filter
    * \param length length of filter (i.e. \a N)
    * \return masked image
    */	
   static DLImage dlMask(const DLImage& image, double* mask[], int length);

   private:

};//end class

#endif //_DLMask_H_
