#ifndef _DLMEDIALAXIS_H_
#define _DLMEDIALAXIS_H_

#include "DLImage.h"



/**
 * \brief Image Medial Axis Transform
 */
class DLMedialAxis
{

   public:
/** \cond **/
   /**
    * Default DLMedialAxis Constructor
    */
   DLMedialAxis();

   /**
    *  DLPoint Copy  Constructor
    */
   DLMedialAxis(const DLMedialAxis & orig);

   /**
    *  Assignment operator
    * \param right 
    */
   const DLMedialAxis & operator=(const DLMedialAxis & right);

   /**
    * Default DLMedialAxis Destructor
    */
   virtual ~DLMedialAxis();
/** \endcond **/

//Arguments: theImage is a pointer to the image to be processed.
//Return Value: CBMedialAxis returns a DLImage object that represents
//the medial axis transform of an image under the chessboard metric.
//Notes: Medial axis transforms are done by taking only those
//pixels which are locally at a maximum distance away from the background.

   /**
    * Returns a DLImage object that represents
    * the medial axis transform of an image under the chessboard metric.<br><br>
    * <b>Note:</b> Medial axis transforms are done by taking only those
    * pixels which are locally at a maximum distance away from the background.
    *
    * \param theImage DLImage that will be transformed
    * \return DLImage An image that has undergone a "Chessboad" Medial Axis 
    * Transformation.
    * \throw DL_NOT_IMPLEMENTED_EXCEPTION when given a color image
    * \see DLSkeleton
    */
   static DLImage dlGetChessboardMedialAxisTransform(const DLImage& theImage);

   /**
    * Returns a DLImage object that represents
    * the medial axis transform of an image under the taxi-cab metric.<br><br>
    * <b>Note:</b> Medial axis transforms are done by taking only those
    * pixels which are locally at a maximum distance away from the background.
    *
    * \param theImage DLImage that will be transformed
    * \return DLImage An image that has undergone a "Taxi-Cab" Medial Axis 
    * Transformation.
    * \throw DL_NOT_IMPLEMENTED_EXCEPTION when given a color image
    * \see DLSkeleton
    */
   static DLImage dlGetTaxiCabMedialAxisTransform(const DLImage & theImage);

   private:

   static int dlGetChessboardDistance(const DLImage& theImage,int xCoord,int yCoord);
   static int dlGetTaxiCabDistance(const DLImage & theImage,int xCoord,int yCoord);
   static bool outOfBounds(int xCoord, int yCoord, int picHeight, int picWidth);
   static bool whitePixOnBdr(const DLImage& theImage,int xCoord,int yCoord,int dist);

};//end class

#endif //_DLMEDIALAXIS_H_
