#ifndef _DLPolar_H_
#define _DLPolar_H_

#include "DLImage.h"
#include "DLPoint.h"
#include "DLBitsPerPixelConverter.h"

#include <cmath>

/**
 * \brief Cartesian-to-polar conversion.
 *
 * The algorithm is implemented as a static function, so no object of
 * class DLPolar needs to be instantiated. See below for a usage example.
 */
class DLPolar
{
   public:
   /** \cond **/
   /**
    * Default DLPolar Constructor
    */
   DLPolar();

   /**
    * Default DLPolar Destructor
    */
   virtual ~DLPolar();
   /** \endcond **/

   /**
    * Compute the polar image of an image. 
    * The origin of the image is its centroid as computed by 
    * DLImage::dlCentroid().
    * The following code example will
    * return the polar image for <i>test.tif</i>:
    * \code
    * DLImage i ("test.tif");
    * DLImage polarImage = DLPolar::dlPolarImage(i, true);
    * \endcode
    * \param  in const DLImage &
    * \param  flag \li \c true (the default) computes distances
    *                  until the corners of the image
    *              \li \c false computes distances until the farthest
    *                  nonwhite pixel from the centroid 
    * \return polar coordinate  image
    */
   static DLImage dlPolarImage ( const DLImage & in, bool flag = true );

   private:

   static DLImage dlPolarImageBinary (  DLImage &, bool flag = true );
   static DLImage dlPolarImageColor  (  DLImage &, bool flag = true );

   static DLImage dlPolarImageGray   (  DLImage &, bool flag = true );
   static DLImage dlPolarImageGray( DLImage & in, DLPoint centroid, double rmax);

};//end class

#endif //_DLPolar_H_
