#ifndef _DLProfile_H_
#define _DLProfile_H_

#include "DLImage.h"
#include "DLBitsPerPixelConverter.h"

#include <cmath>

/**
 * \brief Calculate a projection profile of an image.
 */
class DLProfile
{
   public:
   /** \cond **/
   /**
    * Default DLProfile Constructor
    */
   DLProfile();

   /**
    * Default DLProfile Destructor
    */
   ~DLProfile();
   /** \endcond **/

   /**
    * A selection of possible directions.
    */
	enum DLProjectionDirection {
		DL_HORIZONTAL = 1,
		DL_VERTICAL = 2
	};

   /**
    * Computes a vertical or horizontal projection of the image.
    * Each entry in the array returned will contain the sum of pixel values
    * in an entire row or column; for a vertical projection, each entire column's
    * pixel values are summed, and for a horizontal projection, each entire row's
    * pixel values are summed.
    * \param image DLImage to project
    * \param direction direction to project the image: can be one of
    * \arg DLProfile::DL_HORIZONTAL
    * \arg DLProfile::DL_VERTICAL
    * \return vector of projected image values
    */
   static vector<int> dlProjectProfile ( const DLImage & image, DLProjectionDirection direction );

   /**
    * Computes a horizontal projection of the image.
    * Each entry in the array returned will contain the sum of pixel values
    * in an entire row.
    * Currently only supports 8-bit (grayscale) images
    * \param image DLImage to project
    * \return vector of projected image values
    */
   static vector<int> dlProjectHorizontalProfile ( const DLImage & image );

   /**
    * Computes a vertical projection of the image.
    * Each entry in the array returned will contain the sum of pixel values
    * in an entire column.
    * Currently only supports 8-bit (grayscale) images
    * \param image DLImage to project
    * \return vector of projected image values
    */
   static vector<int> dlProjectVerticalProfile ( const DLImage & image );

   private:

};//end class

#endif //_DLProfile_H_
