#ifndef _DLSharpen_H_
#define _DLSharpen_H_

#include "DLImage.h"
#include "DLPoint.h"
#include "DLBitsPerPixelConverter.h"

#include <cmath>

/**
 * \brief Sharpen an image.
 */
class DLSharpen
{
   public:
   /** \cond **/
   /**
    * Default DLSharpen Constructor
    */
   DLSharpen();

   /**
    * Default DLSharpen Destructor
    */
   virtual ~DLSharpen();
   /** \endcond **/

/**
 * Sharpen an image.
 * This algorithm is implemented as a static function, so no object of
 * class DLSharpen needs to be instantiated. See below for a usage example:
 * \code
 * DLImage i ("test.tif");
 * DLImage sharpenedImage = DLSharpen::dlSharpen(i);
 * \endcode
 *
 * This function (currently) requires a color image as input.
 * At each application of the sharpening operation, a 5x5 sharpening
 * filter is convolved with the image:
 * \f[ \left[
 * \begin{array}{rrrrr}
 * 0 & 0 & -1 & 0 & 0 \\
 * 0 & -8 & -21 & -8 & 0 \\
 * -1 & -21 & 299 & -21 & -1 \\
 * 0 & -8 & -21 & -8 & 0 \\
 * 0 & 0 & -1 & 0 & 0
 * \end{array} \right] \f] 
 * \param in DLImage to sharpen
 * \param numberOfTimes number of times to apply sharpening operation
 * \return sharpened image
 */	
   static DLImage dlSharpen ( const DLImage & in, int numberOfTimes );

   private:

};//end class

#endif //_DLSharpen_H_
