#ifndef _DLTextImage_H_
#define _DLTextImage_H_

#include "DLTextImage.h"
#include "DLImage.h"

#include <iostream>
#include <string>
using namespace std;

/**
 * \brief DLImage representation of a text image
 *
 * DLTextImage is a DLImage that represents a text or document image.  It makes the assumption
 * that a text image would have more white pixels (background) than black pixels.  When creating
 * a text image, it scans through the image and determine which color has more pixels, 
 * white or black.  If image contains more black pixels, DLTextImage will invert
 * the pixels, ensuring a white background and black text.
 *
 * This object only supports binary (black and white) images.  User must 
 * downscale the image to binary before calling this function.
 * (See DLBitsPerPixelConverter.)
 * Will throw an exception if not a binary image.
 *
 */

class DLTextImage : public DLImage 
{
   public:

   /**
    * Constructor: loads an  image with 
    * the (optional) image type specified by passing a
    * pointer to one of the global image format singleton objects.
    * If no image type is provided,
    * DOCLIB will identify the image by the file extension and/or
    * by the image file content.
    *
    * Example:
    * \code
    * DLTextImage ti("doc.xyz", DLTIFFImage::createImageReader());
    * \endcode
    * forces \c doc.xyz to be loaded as a TIFF file.
    * \param filename name of file to be opened
    * \param  imageType image format object pointer.
    * This could be any instance of the supported image types. 
    */
   DLTextImage(const char *filename, DLBaseImage* imageType = NULL);

   private:
   /**
    * dlHasMoreBlackPixel 
    */
   bool dlHasMoreBlackPixel();

   ///Fix bit
   void dlReverseDocumentPixel();

};//end class


#endif // _DLIMAGE_H_
