/*
  --------------------------------------------------------------------------------
  -   Module      :   mem_file.h
  -   Description :   A general purpose library for manipulating a memory area
  -                   as if it were a file.
  -                   mfs_ stands for memory file system.
  -   Author      :   Mike Johnson - Banctec AB 03/07/96
  -                   
  --------------------------------------------------------------------------------
*/

/* 

Copyright (c) 1996 Mike Johnson
Copyright (c) 1996 BancTec AB

Permission to use, copy, modify, distribute, and sell this software
for any purpose is hereby granted without fee, provided
that (i) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, and (ii) the names of
Mike Johnson and BancTec may not be used in any advertising or
publicity relating to the software.

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  

IN NO EVENT SHALL MIKE JOHNSON OR BANCTEC BE LIABLE FOR
ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
OF THIS SOFTWARE.

*/

#ifndef __mfs_file_c
#define __mfs_file_c

/*
  --------------------------------------------------------------------------------
  -   Includes
  --------------------------------------------------------------------------------
*/

#include    <cstdio>
#include    <cstdlib>
#include    <cstring>

/*
  --------------------------------------------------------------------------------
  -   Definitions
  --------------------------------------------------------------------------------
*/

static const int MAX_BUFFS  = 20;

static const int  MFS_EINVAL = 22;
static const int  MFS_EMFILE = 24;
static const int  MFS_EBADF = 77;
static const int  MFS_EDQUOT = 122;


/*
  --------------------------------------------------------------------------------
  -   Function prototypes
  --------------------------------------------------------------------------------
*/

int mfs_open (void *ptr, int size, const char *mode);
int mfs_lseek (int fd, int offset, int whence);
int mfs_lseekForTIFFRead (int fd, int offset, int whence);
int mfs_read (int fd, void *buf, int size);
int mfs_write (int fd, void *buf, int size);
int mfs_size (int fd);
int mfs_map (int fd, char **addr, size_t *len);
int mfs_unmap (int fd);
int mfs_close (int fd);
int extend_mem_file (int fd, int size);
void mem_init ();

/*
  --------------------------------------------------------------------------------
  -   Function    :   mfs_gets()
  -
  -   Arguments   :   File descriptor, buffer, size
  -
  -   Returns     :   as per man gets (2)
  -
  -   Description :   Does the same as gets (2) except on a memory based file.
  -                   Note: An attempt to read past the end of memory currently
  -                   allocated to the file will return 0 (End Of File)
  -                   
  --------------------------------------------------------------------------------
*/

int mfs_gets (int fd, void *clnt_buf, int size);

#endif
