#include "DLCommandLineParser.h"
#include <string>
#include <fstream>
#include <list>
using namespace std;


void console();
void showUsage();
void Tokenize(const string& str,
              list<string>& tokens,
              const string& delimiters);

int main(int argc, char *argv[]) 
{

   try {

      if ( argc < 2 )
      {
         showUsage(); 
      }

      else if ((argc == 3) && strcmp(argv[1], "-script") == 0)
      {

         ifstream infile(argv[2]) ; 
         if(infile)
         {
            list <string> cmdVector;
            string cmdStr = "";
            while (!infile.eof())
            {
               cmdStr = "";
               infile >> cmdStr;
               if(cmdStr.size() > 0)
               {
                  if(cmdStr[0] == '#')
                  {
                     infile.ignore(80, '\n');
                  }
                  else
                  {
                     cmdVector.push_back(cmdStr);
                     cout << cmdStr << " ";
                  }
               }
            }
            infile.close();	
            cout << endl;

            DLCommandLineParser dlc;
            dlc.processCommand(cmdVector);
         }
         else
         {
            cout << "Error Opening Config File : " << argv[2] << endl;
         }
      }

      else if ((argc == 2) && strcmp(argv[1], "-console") == 0)
      {
         console();
      }

      else
      {
         list <string> cmdVector;
         string cmdStr = "";
         list<string> outputCommand;

         for ( int idx = 1; idx < argc; idx++ ) {
            cmdStr = argv[idx];
            if(cmdStr == "-output")
            {
               outputCommand.push_back(cmdStr);
               idx++;
               outputCommand.push_back(argv[idx]);
            }
            else
            {
               cmdVector.push_back(cmdStr);
            }

         }


         DLCommandLineParser dlc;
         dlc.processCommand(cmdVector);
         if(outputCommand.size() != 0)
         {
            dlc.processCommand(outputCommand);

         }
      }
	  
   } catch(DLException E) {

      // no need to print error message again--has already been printed in DLCommandLineParser
      //cout <<  "Exception: " << E.dlGetErrorMsg() << endl;
      return 1;

   } catch(exception E) {

      cout << "Exception: " << E.what() << endl;
      return 1;

   } catch(exception *E) {

      cout << "Exception: " << E->what() << endl;
      delete E;
      return 1;

   } catch(...) {

      cout << "ERROR: Unknown Exception!!!" << endl;
      return 1;

   }

   // no error--return zero
   return 0;
}

void console()
{
   string cmd = "";
   char cmdChars[80];
   DLCommandLineParser dlc;
   cout << "DOCLIB Command line.  \nType \"quit\" to Exit\nType \"help\" for command list.\n";
   list <string> cmdList;
   while (cmd != "quit")
   {
      cout << ">"; 
      cin.getline(cmdChars, 80);
      cmd = cmdChars;
      if(cmd == "help" || cmd == "Help")
      {
         showUsage();
      }
      else if (cmd == "quit" || cmd == "Quit")
      {
         continue;
      }
      else
      {
         try{
            cmdList.clear();
            Tokenize(cmd, cmdList, " ");
            dlc.processCommand(cmdList);
         }
         catch(DLException &E)
         {
            cout << E.what() << endl;
         }
         catch (exception E)
         {
            cout << E.what() << endl;

         }
         catch (exception *E)
         {
            cout << E->what() << endl;

         }
         catch (...)
         {
            cout << "unknown error!!\n";
         }

      }
   }
}

/**
 * Tokenize takes a string and split the string by a token
 * \param str string to tokenize
 * \param tokens stores the result of tokens
 * \param delimiters delimiter use to divide the string
 **/
void Tokenize(const string& str,
              list<string>& tokens,
              const string& delimiters)
{
   // Skip delimiters at beginning.
   string::size_type lastPos = str.find_first_not_of(delimiters, 0);
   // Find first "non-delimiter".
   string::size_type pos     = str.find_first_of(delimiters, lastPos);

   while (string::npos != pos || string::npos != lastPos)
   {
      // Found a token, add it to the vector.
      tokens.push_back(str.substr(lastPos, pos - lastPos));
      // Skip delimiters.  Note the "not_of"
      lastPos = str.find_first_not_of(delimiters, pos);
      // Find next "non-delimiter"
      pos = str.find_first_of(delimiters, lastPos);
   }
}

void showUsage()
{
   cout << "DoclibCommandLine -script scriptfile" << endl;
   cout << "DoclibCommandLine -input imagefile command [-args]" << endl;
   cout << "DoclibCommandLine -console" << endl << endl;
 
   cout << "<Options>\n";
   cout << "-input input_file" << endl;
   cout << "-loadDocFile input_file" << endl;
   cout << "-resize factor(i.e. 0.1, 0.2 ... 0.9)" << endl;
   cout << "-resize width height" << endl;
   cout << "-output filename" << endl;
   cout << "-rotate angle" << endl;
   cout << "-subimage x y height width" << endl;
   cout << "-flip direction(DL_HORIZONTAL|DL_VERTICAL)" << endl;
   cout << "-contour" << endl;
   cout << "-reverse" << endl;
   cout << "-paste imagefilename x y" << endl;
   cout << "-project direction(DL_HORIZONTAL|DL_VERTICAL)" << endl;
   cout << "-addImage imagefilename x y" << endl;
   cout << "-convert conversionCode(DL_2BINARY|DL_2GRAY|DL_2COLOR)" << endl;
   cout << "-boundCC" << endl;
   cout << "-filterCCLessThan massSize" << endl;
   cout << "-filterCCDensityLessThan densitySize" << endl;
   cout << "-filterCCDensityGreaterThan densitySize" << endl;
   cout << "-setPixel x y r g b" << endl;
   cout << "-drawLine x1 y1 x2 y2" << endl;
   cout << "-drawLine DLColorType x1 y1 x2 y2" << endl;
   cout << "-drawBox x1 y1 x2 y2" << endl;
   cout << "-drawBox DLColorType(DL_RED, DL_GREEN, DL_BLUE) x1 y1 x2 y2" << endl;
   cout << "-dilate numberOfTimes" << endl;
   cout << "-erode numberOfTimes" << endl;
   cout << "-sharpen numberOfTimes" << endl;
   cout << "-blur numberOfTimes" << endl;

   
   // cout << "-gaussianBlur numberOfTimes" << endl;

   //cout << "-mask factor" << endl;
   cout << "-toYCrCb" << endl;
   cout << "-dlYCrCbBinarization" << endl;
   cout << "-dl256Color quantized (1|0)" << endl;
   cout << "-dlPercentThresholdBinarization percent (i.e 90 or .9)" << endl;
   cout << "-dlThresholdBinarization rThresh gTresh bThresh" << endl;
   cout << "-dlColor2Gray_global" << endl;
   cout << "-dlGray2Binary_global thresh" << endl;
   cout << "-dlNIBlackBinarization" << endl;
   cout << "-dlBinary2Color" << endl;
   cout << "-dlGray2Color" << endl;
   cout << "-dlBinary2Gray" << endl;
   cout << "-dlSkeletonize" << endl;
   cout << "-polar TRUE|FALSE" << endl;
   cout << "-cartesian" << endl;
   cout << "-normalize" << endl;
   cout << "-deskew" << endl;
   cout << "-centroid" << endl;
   cout << "-logger DEBUG|INFO|CRITICAL|OFF" << endl;
   cout << "-dbg_loadAsMemory YES|NO    Debug command forces the DLImage to load"
        << "     the image file into memory then load image as a memory block" << endl;
   cout << "-imageInfo" << endl;
}
/************************************************************************ *
FILE: $RCSfile: DLCommandLine.cpp,v $ 
DESCRIPTION: 

* ************************************************************************ *
Modification Log: $Log: DLCommandLine.cpp,v $
Modification Log: Revision 1.13  2006/08/29 19:46:22  meyers_tim
Modification Log: ran through emacs autoindent
Modification Log:
Modification Log: Revision 1.12  2006/08/28 20:25:08  meyers_tim
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.11  2006/05/17 19:16:57  meyers_tim
Modification Log: Added ability to turn off Logger Warnings
Modification Log:
Modification Log: Revision 1.10  2006/03/08 21:17:25  meyers_tim
Modification Log: Adding new features
Modification Log:
Modification Log: Revision 1.9  2006/03/07 21:39:05  meyers_tim
Modification Log: Added DLNormalize and DLPolar objects
Modification Log:
Modification Log: Added their functionality to the regression tests.
Modification Log:
Modification Log: Updated GroundTruth Data
Modification Log:
Modification Log: Revision 1.8  2006/02/27 23:19:46  ramsbrock_daniel
Modification Log: Fixed return value to reflect error condition
Modification Log:
Modification Log: Revision 1.7  2006/02/10 18:11:34  chen_kevin
Modification Log: Added "-imageInfo" to print the current image header info
Modification Log:
Modification Log: Revision 1.6  2005/11/10 15:04:04  chen_kevin
Modification Log: Added addImage command to commandline
Modification Log:
Modification Log: Revision 1.5  2005/09/06 19:57:39  chen_kevin
Modification Log: changed DlGetErrMsg to dlGetErrorMsg
Modification Log:
Modification Log: Revision 1.4  2005/06/29 21:52:23  chen_kevin
Modification Log: Added the ability to force the commandline object to use memory based loading.
Modification Log:
Modification Log: Revision 1.3  2005/06/27 19:30:46  chen_kevin
Modification Log: Added command to turn on debugger on command line.
Modification Log:
Modification Log: Revision 1.2  2005/06/16 15:33:02  chen_kevin
Modification Log: Updated DLCommandLine to only allow one output statement for commond line mode.
Modification Log:
Modification Log: Revision 1.1  2005/04/11 17:54:39  chen_kevin
Modification Log: This file was moved from ..\doclib\DoclibTools\DoclibCommandLine directory back in DOCLIB_before_reorg_4_11 tag.  To access the old change logs, please checkout tag version of DOCLIB "DOCLIB_before_reorg_4_11 tag"
Modification Log:
Modification Log: Revision 1.16  2005/04/01 19:32:06  meyers_tim
Modification Log: Added centroid calculation.  command is:
Modification Log: -centroid
Modification Log:
Modification Log: Revision 1.15  2005/02/02 16:58:41  chen_kevin
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.14  2005/01/21 16:02:18  chen_kevin
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.13  2005/01/14 18:41:58  chen_kevin
Modification Log: Added new command to doclib commandline.
Modification Log:
Modification Log: Revision 1.12  2004/11/08 20:38:01  sdsampa
Modification Log: *** empty log message ***
Modification Log:
Modification Log: Revision 1.11  2004/11/02 14:37:43  chen_kevin
Modification Log: Added color support for -drawLine
Modification Log:
Modification Log: Revision 1.10  2004/10/28 04:07:08  sdsampa
Modification Log: support dlDrawBox
Modification Log:
Modification Log: Revision 1.9  2004/10/26 02:35:58  sdsampa
Modification Log: support dlDrawBox
Modification Log:
Modification Log: Revision 1.8  2004/09/20 03:06:25  zhugy
Modification Log: Restore -deskew option to the commandline for image deskewing
Modification Log:
Modification Log: Revision 1.7  2004/08/02 14:13:31  chen_kevin
Modification Log: Added logic to ignore lines with # as the first character.
Modification Log:
Modification Log: Revision 1.6  2004/07/12 23:15:32  reddyc
Modification Log: Implemention of the skeletonization algorithim from:
Modification Log: http://www.csc.liv.ac.uk/~prima/ICDAR2003/Papers/0143_512_huang_l.pdf
Modification Log:
Modification Log: Added some changes to the DLCommandLine in the protoType folder, not done yet.
Modification Log:
Modification Log: Revision 1.5  2004/07/06 14:53:24  sdsampa
Modification Log: update cvs log
Modification Log: 

* ************************************************************************ *
DocLib is being developed under contract by a collaboration between:

The Laboratory for Language and Media Processing
Unviersity of Maryland, College Park
and 
Booz Allen Hamilton

All Rights Reserved, 2003
* *************************************************************************/
